

/***************************************************************************************
*
* Rutinas para el manejo del puerto serie de un PC a cualquier velocidad
*
* Adaptadas por Ibercomp S.R.L. - Mayo de 1993
*
***************************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>                        /* getvect()... */
#define BIN_READ    "rb"

#define VERSION  "V1.00"                /* Versin de este modulo */

typedef unsigned int uint;              /* definicion de algunos tipos */
typedef unsigned char uchar;

/*********** RX por buffer ************************************************************/
#define RX_BUF_SIZE     512             /* Tamao del buffer en potencia de 2 */
static uint ser_rinx=0;                 /* Indice de entrada RX-Serie */
static uint ser_routx=0;                /* Indice de salida  RX-Serie */
static uchar ser_rx_buf[RX_BUF_SIZE];   /* Tamao del buffer de recepcin RX */

/**************************************************************************************
*
* Manejador de puerto serie PC
*
* Informacin de los registros del PC-SIO  ej eb: C t 5/88  pg. 204
*
* Fecha      : 10. 12. 1992
* Compilador : Borland C++ 3.1
*
* Recepcin por interrupciones, escritura directa al SIO.
*
***************************************************************************************/

#define IRQ_CTRL        0x20            /* Controlador de interrupciones */
#define IRQ_MASKE       0x21            /* Puerto para la mscara de interrupciones */
#define SIO_EIN         0xE7            /* Bits del SIO en el controlador. */
#define EOI             0x20            /* Fin de interrupcin */

#define COM1            0x3F8           /* Direccin a la base de COM1 */
#define COM2            0x2F8           /* Direccin a la base de COM2 */
                                        /* Offsets relativos de los registros: */
#define IER             1               /* Registro de habilitacin de interrupcin. */
#define IIR             2               /* Registro de identificacin de interrupcin */
#define LCR             3               /* Registro de control de linea */
#define MCR             4               /* Registro de control de MODEM */
#define LSR             5               /* Registro de estado de la linea */
#define MSR             6               /* Registro de estado del MODEM */

#define COM1_VEC        0xC             /* Vector SIO1 */
#define COM2_VEC        0xB             /* Vector SIO2 */
#define TIMER_VEC       0x1C            /* Vector del temporizador*/

static int com=COM1;                    /* Direccin de la base */
static int com_vec=COM1_VEC;            /* Direccin de la interrupcin */

static uint no_sio_mask;                /* Antigua mscara de interrupcin */
static void interrupt (*o_com_vec)();   /* Antiguo vector del SIO */
static void interrupt (*o_timer_vec)(); /* Antiguo vector del temporizador */

#define TIMEOUT 10                      /* Debe ser rpido */
static volatile int klick_cnt=-1;       /* Contador de clicks */

int hw_test(void);                      /* Borland C: Prototipos */
int reset(void);

/*********** Interrupcin del temporizador ********************************************/
static void interrupt timer(){          /* Contador del sistema */
    if(klick_cnt>=0) klick_cnt--;       /* Se decrementa 18 veces por segundo */
}

/*********** Interrupcin de transferencia: Los datos de entrada son buffereados ******/
static void interrupt transfer(){

  ser_rx_buf[ser_rinx++]=inportb(com);
  ser_rinx &= (RX_BUF_SIZE-1);          /* Buffer ciclico - anillo */
  klick_cnt=TIMEOUT;                    /* Espera de 0.25 ms */
                                        /* 32: seala buffer de transmisin vacio */
  if(inportb(com+LSR) & (2+4+8+16)){    /****  Hallado un error ? ****/
    inportb(com);
    inportb(com);
    inportb(com);
    inportb(com);
  }
  outportb(IRQ_CTRL,EOI);               /* Arranca interrupcin! */
}
/******************* seriell_put_byte() ***********************************************/
void envia_rs232c(register uint obyte){
  while(!(inportb(com+LSR) & 32));      /* Espera a que se envi el byte anterior! */
  outportb(com,obyte);                  /* emitir caracter */
}

/******* Enviar ***********************************************************************/
void envia_bloque(int typ, uchar* source, uint len){
  uchar tx_fcs=0;                       /* byte de comprobacin checksum */
  int i;

  disable();                            /* Antes de transmitir vaciar buffer */
  ser_rinx=ser_routx;
  enable();
  for(i=0;i<4;i++) inportb(com);        /* vaciar SIO */

  while(!(inportb(com+LSR) & 32));      /* Espera que el byte antiguo se halla enviado */
  outportb(com,typ);                    /* Envia un caracter */

  while(len--){
    while(!(inportb(com+LSR) & 32));    /* Espera que se halla enviado el byte antiguo */
    outportb(com,*source);              /**** Envia un caracter ****/
    tx_fcs += *source++;                /* Incrementa el contador del checksum */
  }
  while(!(inportb(com+LSR) & 32));      /* Espera a que se halla enviado el byte */
  outportb(com,tx_fcs);                 /* Emite la suma */
}

/******* Leer *************************************************************************/

/******** Hay algo en le buffer? *****************************************************/
int estado_rs232c(void){
  register uint delta;
  disable();
  delta=ser_rinx-ser_routx;
  enable();
  return delta?1:0;                     /* No hay nada = 0, Hay algo = 1*/
}

/********* Leer un byte ***************************************************************/
int recibe_rs232c(void){
  uchar ibyte;

	if(!estado_rs232c()){
		klick_cnt=TIMEOUT;
		do{
			if(klick_cnt<0) return -1;        /* -1 Timeout */
		}while(!estado_rs232c());       /* Espera mientras no haya nada */
  }
  ibyte=ser_rx_buf[ser_routx++];        /* Recoge un caracter */
  ser_routx &= (RX_BUF_SIZE-1);         /* Calculado el modulo del buffer */
  return ibyte;                         /* Devuelve caracter leido */
}

/******* recibe_bloque(): Rutina central de lectura ***********************************/

int recibe_bloque(int typ, uchar* dest, uint len){
  uchar rx_fcs=0;                       /* pone checksum a cero */
  register int inp;

  /* Primero identifica la cabecera */
  if((inp=recibe_rs232c())<0) return -1;/* -1: Timeout - Typ */
  if(inp!=typ) return -2;               /* -2: Typ - Error */

  while(len--){
    if((inp=recibe_rs232c())<0) return -3; /* -3: Timeout - en datos */
    rx_fcs+=(uchar)inp;                 /* FCS incrementa checksum */
    *dest++=(uchar)inp;                 /* guarda en posicion */
  }
  if((inp=recibe_rs232c())<0) return -4;/* -4: Timeout checksum */
  if(rx_fcs!=(uchar)inp) return -5;     /* -5: ERROR checksum */
  return 0;                             /* Al final OK! */
}

/******* inicia_rs232c() Inicializa puerto serie: 1,2, Baudrate ***********************/
void inicia_rs232c(int com_nr,long baudrate){
  uint divisor;
  int i;

  if(com_nr<1 || com_nr>2){
    puts("\007*** Error: Nmero del puerto serie incorrecto");
    exit(-1);
  }
  if(baudrate>115200L || baudrate<1L){
    printf("\007*** Error: No es posible seleccionar %ld baudios!\n",baudrate); 
    exit(-1);
  }

  com=(com_nr-1)?COM2:COM1;             /* Selecciona direccin base correcta */
  com_vec=(com_nr-1)?COM2_VEC:COM1_VEC;
  divisor=(uint)(115200L/baudrate);

  no_sio_mask=inportb(IRQ_MASKE) | (~SIO_EIN);  /* No dispone de  SIO! */
  outportb(IRQ_MASKE,no_sio_mask);

  outportb(com+MCR,0);                  /* Borra registros */
  outportb(com+MSR,0);

  o_com_vec=getvect(com_vec);           /* Obtiene vector antiguo */
  setvect(com_vec,transfer);            /* Pone vector nuevo */
  o_timer_vec=getvect(TIMER_VEC);
  setvect(TIMER_VEC,timer);

  outportb(com+LCR,0x80);               /* Habilita el bit del lach divisor */
  outportb(com,divisor&255);            /* Divisor Bajo */
  outportb(com+1,divisor>>8);           /* Divisor Alto */
  outportb(com+LCR,3);                  /* 8 Bits, 1 Stopbit, sin paridad */

  outportb(com+IER,1);                  /* Solo Irq de entrada*/
  outportb(com+MCR,0xB);                /* Activa IRQ, RTS=DTR=1 */
  enable();                             /* Interrupciones siempre activas */
  for(i=0;i<20;i++) inportb(com);       /* Limpia porquera */
  outportb(IRQ_MASKE, no_sio_mask & SIO_EIN); /* Envia nueva mscara */
}

/******** Desinicializa el puerto serie ***********************************************/
void termina_rs232c(void){
  setvect(com_vec,o_com_vec);           /* Pone antiguo manejador */
  setvect(TIMER_VEC,o_timer_vec);       /* Pone antiguo temporizador */
  outportb(IRQ_MASKE,no_sio_mask);      /* Pone antigua mscara */
}

